@@ -3,7 +3,7 @@
 
 
 package BeamerReveal::FrameConverter;
-our $VERSION = '20251227.1426'; # VERSION
+our $VERSION = '20251230.2042'; # VERSION
 
 use strict;
 use warnings;
@@ -14,24 +14,29 @@ use File::Which;
 use File::Path;
 
 use BeamerReveal::IPC::Run;
+use IPC::Run qw(harness start pump finish); 
+
+use BeamerReveal::Log;
 
 sub nofdigits { length( "$_[0]" ) }
 
 
 sub new {
   my $class = shift;
-  my ( $base, $pdffile, $xres, $yres ) = @_;
+  my ( $base, $pdffile, $xres, $yres, $progressId ) = @_;
 
   my $self = {
 	      base => $base,
 	      xres => $xres,
 	      yres => $yres,
-	      file => $pdffile };
+	      file => $pdffile,
+	      progressId => $progressId,
+	     };
   $class = (ref $class ? ref $class : $class );
   bless $self, $class;
   
   $self->{pdftoppm} = File::Which::which( 'pdftoppm' )
-    or die( "Error: your setup is incomplete, I cannot find pdftoppm (part of the poppler library)\n" .
+    or $Beamer::Log::logger->fatal( "Error: your setup is incomplete, I cannot find pdftoppm (part of the poppler library)\n" .
 	    "Install 'Poppler-utils' and make sure pdftoppm is accessible in a directory on your PATH list variable\n" );
 
   #  $self->{slides} = File::Spec->catfile( $self->{base}, 'media', 'Slides' );
@@ -57,8 +62,23 @@ sub toJPG {
 	      '-jpegopt',
 	      'optimize=y,quality=85',
 	      '-scale-to-x', @{[1.5*$self->{xres}]},
-	      '-scale-to-y', @{[1.5*$self->{yres}]} ];
-  BeamerReveal::IPC::Run::run( $cmd, 0, 2 );
+	      '-scale-to-y', @{[1.5*$self->{yres}]},
+	      '-progress',
+	    ];
+  #  BeamerReveal::IPC::Run::run( $cmd, 0, 2 );
+  
+  my $logger = $BeamerReveal::Log::logger;
+  BeamerReveal::IPC::Run::runsmart( $cmd, 2, qr/^(\d+) (\d+) .*$/,
+				    sub {
+				      while( scalar @_ ) {
+					my ( $a, $b ) = ( shift @_, shift @_ );
+					$logger->progress( $self->{progressId},
+							   $a, "background $1/$2", $b );
+				      }
+				    },
+				    0, # coreId
+				    2, # indent
+				  );
 }
 
 1;
@@ -75,7 +95,7 @@ BeamerReveal::FrameConverter - FrameConverter
 
 =head1 VERSION
 
-version 20251227.1426
+version 20251230.2042
 
 =head1 SYNOPSIS
 
@@ -113,7 +133,7 @@ the frame converter
 
 =head2 $fc->toJPG()
 
-Converts a pdf-file to one jpg file per frame. This is done at 4/3 of the canvas resolution.
+Converts a pdf-file to one jpg file per frame. This is done at 3/2 of the canvas resolution.
 
 =head1 AUTHOR
 
