@@ -3,7 +3,7 @@
 
 
 package BeamerReveal::MediaManager;
-our $VERSION = '20251227.1426'; # VERSION
+our $VERSION = '20251230.2042'; # VERSION
 
 use strict;
 use warnings;
@@ -23,9 +23,15 @@ use IO::File;
 
 use MCE::Hobo;
 use MCE::Util;
+use MCE::Shared::Scalar;
+
+use Time::HiRes;
 
 use BeamerReveal::TemplateStore;
 use BeamerReveal::IPC::Run;
+use IPC::Run qw(harness start pump finish); 
+
+use BeamerReveal::Log;
 
 sub min { $_[$_[0] > $_[1] ] }
 sub nofdigits { length( "$_[0]" ) }
@@ -36,12 +42,17 @@ sub new {
   my ( $jobname, $base, $presentationprms ) = @_;
 
   my $self = {
-	      jobname => $jobname,
-	      base    => $base
+	      jobname    => $jobname,
+	      base       => $base,
 	     };
   $class = (ref $class ? ref $class : $class );
   bless $self, $class;
 
+  $self->{presentationparameters} = $presentationprms;
+
+  ########################
+  # Prepare all the paths
+
   $self->{videos}     = "$self->{base}/media/Videos";
   $self->{audios}     = "$self->{base}/media/Audios";	
   $self->{images}     = "$self->{base}/media/Images";	
@@ -49,19 +60,11 @@ sub new {
   $self->{iframes}    = "$self->{base}/media/Iframes";	
   $self->{slides}     = "$self->{base}/media/Slides";
   $self->{reveal}     = "$self->{base}/libs";
-  # $self->{videos}     = "$self->{base}, 'media', 'Videos' );
-  # $self->{audios}     = File::Spec->catfile( $self->{base}, 'media', 'Audios' );	
-  # $self->{images}     = File::Spec->catfile( $self->{base}, 'media', 'Images' );	
-  # $self->{animations} = File::Spec->catfile( $self->{base}, 'media', 'Animations' );	
-  # $self->{iframes}    = File::Spec->catfile( $self->{base}, 'media', 'Iframes' );	
-  # $self->{slides}     = File::Spec->catfile( $self->{base}, 'media', 'Slides' );
-  # $self->{reveal}     = File::Spec->catfile( $self->{base}, 'libs' );
-  $self->{presentationparameters} = $presentationprms;
   
-  # create animations, but don't remove them
+  # create animation path, but don't remove contents
   File::Path::make_path( $self->{animations} );
 
-  # create all the rest, but also clean them
+  # create all the ohter paths and also clean them
   for my $item ( qw(reveal videos audios iframes images) ) {
     File::Path::rmtree( $self->{$item} );
     File::Path::make_path( $self->{$item} );
@@ -74,10 +77,12 @@ sub new {
     $realTexFileName = $texFileName . ".$ext";
     last if ( -r $realTexFileName );
   }
+
+  my $logger = $BeamerReveal::Log::logger;
   
   my $texFile = IO::File->new();
   $texFile->open( "<$realTexFileName" )
-    or die( "Error: could not open your original LaTeX source file '$realTexFileName'\n" );
+    or $logger->fatal( "Error: could not open your original LaTeX source file '$realTexFileName'\n" );
   $self->{preamble} = '';
   my $line;
   do { $line = <$texFile> } until( $line =~ /\\usepackage[^{]*{beamer-reveal}/ );
@@ -89,21 +94,29 @@ sub new {
   $texFile->close();
   $self->{preamble} .= "%% End of preamble excerpt\n";
 
+  ####################################################
+  # check all the preconditions for running our tools
   $self->{compiler} = File::Which::which( $self->{presentationparameters}->{compiler} )
-    or die( "Error: your setup is incomplete, I cannot find your $self->{presentationparameters}->{compiler } compiler (should be part of your TeX installation)\n" .
+    or $logger->fatal( "Error: your setup is incomplete, I cannot find your $self->{presentationparameters}->{compiler } compiler (should be part of your TeX installation)\n" .
 	    "Make sure it is accessible in a directory on your PATH list variable\n" );
   
   $self->{pdftoppm} = File::Which::which( 'pdftoppm' )
-    or die( "Error: your setup is incomplete, I cannot find pdftoppm (part of the poppler library)\n" .
+    or $logger->fatal( "Error: your setup is incomplete, I cannot find pdftoppm (part of the poppler library)\n" .
 	    "Install 'Poppler-utils' and make sure pdftoppm is accessible in a directory on your PATH list variable\n" );
 
   $self->{pdfcrop} = File::Which::which( 'pdfcrop' )
-    or die( "Error: your setup is incomplete, I cannot find pdfcrop (should be part of your TeX installation)\n" .
+    or $logger->fatal( "Error: your setup is incomplete, I cannot find pdfcrop (should be part of your TeX installation)\n" .
 	    "Make sure it is accessible in a directory on your PATH list variable\n" );
   $self->{ffmpeg} = File::Which::which( 'ffmpeg' )
-    or die( "Error: your setup is incomplete, I cannot find ffmpeg\n" .
+    or $logger->fatal( "Error: your setup is incomplete, I cannot find ffmpeg\n" .
 	    "Install 'FFmpeg' (from www.ffmpeg.org) and make sure ffmpeg is accessible in a directory on your PATH list variable\n" );
 
+  ##########################################################################
+  # We sell things, before we make the, so we need to keep a backorder list
+
+  $self->{copyBackOrders}  = [];
+  $self->{constructionBackOrders}  = [];
+  
   return $self;
 }
 
@@ -111,8 +124,6 @@ sub new {
 sub revealToStore {
   my $self = shift;
 
-#  my $revealTree = File::Spec->catfile( File::ShareDir::dist_dir( 'BeamerReveal' ),
-#					'libs' );
   my $revealTree = File::ShareDir::dist_dir( 'BeamerReveal' ) . '/libs';
   my $destTree = $self->{reveal};
 
@@ -125,21 +136,22 @@ sub slideFromStore {
   my ( $slide ) = @_;
   
   # copy file
-#  my $fullpathid = File::Spec->catfile( $self->{slides}, $slide );
   my $fullpathid = "$self->{slides}/$slide";
   # return store id
   return $fullpathid;
 }
 
 
-sub animationToStore {
+sub animationFromStore {
   my $self = shift;
   my ( $animation ) = @_;
-
+  
+  my $logger = $BeamerReveal::Log::logger;
+  
   my $animid  = Digest::SHA::hmac_sha256_hex( $animation->{tex} );
-  #  my $animdir = File::Spec->catfile( $self->{animations}, $animid );
   my $animdir = "$self->{animations}/$animid";
   my $fullpathid = $animdir . ".mp4";
+  
   unless ( -r $fullpathid ) {
     File::Path::make_path( $animdir );
 
@@ -153,22 +165,62 @@ sub animationToStore {
        'ANIMATION' => $animation->{tex},
       };
     my $fileContent = BeamerReveal::TemplateStore::stampTemplate( $tTemplate, $tStamps );
-
+  
     my $nofFrames = floor( $animation->{framerate} * $animation->{duration} );
     my $nofCores  = MCE::Util::get_ncpu();
     if ( $nofCores > 4 ) {
       $nofCores = ceil( $nofCores / 2 );
     }
+
+    push @{$self->{constructionBackOrders}},
+      {
+       animation   => $animation,
+       fileContent => $fileContent,
+       nofFrames   => $nofFrames,
+       nofCores    => $nofCores,
+       animid      => $animid,
+      };
+  }
+  return $fullpathid;
+}
+
+
+sub processConstructionBackOrders {
+  my $self = shift;
+  my ( $progressId ) = @_;
+  
+  my $logger = $BeamerReveal::Log::logger;
+  
+  my $totalNofBackOrders = @{$self->{constructionBackOrders}};
+
+  $logger->progress( $progressId, 1, 'reusing cached data', 1 ) unless $totalNofBackOrders;
+  
+  for( my $i = 0; $i < $totalNofBackOrders; ++$i ) {
+    my $bo = $self->{constructionBackOrders}->[$i];
+
+    my $animation   = $bo->{animation};
+    my $fileContent = $bo->{fileContent};
+    my $nofFrames   = $bo->{nofFrames};
+    my $nofCores    = $bo->{nofCores};
+    my $animid      = $bo->{animid};
+    my $animdir     = "$self->{animations}/$animid";
     
     # I cannot get multithreading/multiprocessing to work reliably on MS-Windows
+    my $progress;
+    my $sliceSize = $nofFrames;
     if ( $^O eq 'MSWin32' ) {
+      $logger->log( 6, "- Preparing media generation of $nofFrames (alas, no parallellization on MS-Windows)" );
       $nofCores = 1;
+      $progress = MCE::Shared::Scalar->new( 0 );
+    }
+    elsif ( $nofCores == 1 ) {
+      $progress = MCE::Shared::Scalar->new( 0 );
+    }      
+    else {
+      $sliceSize = ceil( $nofFrames / $nofCores );
+      $logger->log( 6, "- Preparing media generation of $nofFrames frames in $nofCores threads at $sliceSize frames per thread" );
+      $progress = MCE::Shared->scalar( 0 );
     }
-    my $sliceSize = ceil( $nofFrames / $nofCores );
-
-
-    
-    say STDERR "      - Preparing media generation of $nofFrames frames in $nofCores threads at $sliceSize frames per thread";
 
     # make planning
     my $frameCounter = 0;
@@ -182,19 +234,22 @@ sub animationToStore {
       $plan->{slicestart} = 1;
       $plan->{slicestop}  = $plan->{nstop} - $plan->{nstart};
       $plan->{nstop}      -= 1;
-
+      
       # register plan
       push @$planning, $plan;
 
       # prepare for next core
       $frameCounter += $sliceSize;
     }
-
-    # I cannot get multithreading/multiprocessing to work reliably on MS-Windows
-    if ( $^O eq 'MSWin32' ) {
+    
+    $logger->progress( $progressId, 0, "animation @{[$i+1]}/$totalNofBackOrders",
+		       $nofFrames + $nofCores * 3 );
+    
+    if ( $nofCores == 1 ) {
       # single-threaded
       for( my $i = 0; $i < @$planning; ++$i ) {
-	_animWork( $planning->[$i], $nofCores, $fileContent, $animdir, $self, $animation, $sliceSize );
+	_animWork( $planning->[$i], $nofCores, $fileContent, $animdir, $self, $animation,
+		   $sliceSize, $progress, $progressId );
       }
     }
     else {
@@ -205,12 +260,27 @@ sub animationToStore {
 	  (
 	   sub {
 	     _animWork( @_ )
-	   }, ( $planning->[$i], $nofCores, $fileContent, $animdir, $self, $animation, $sliceSize )
+	   }, ( $planning->[$i], $nofCores, $fileContent, $animdir, $self, $animation,
+		$sliceSize, $progress, $progressId )
 	  );
       }
       
+      my $activeworkers = @hobos;
+      while ( $activeworkers ) {
+	
+	my @joinable = MCE::Hobo->list_joinable();
+	foreach my $hobo ( @joinable ) {
+	  $hobo->join();
+	  --$activeworkers;
+	}
+	
+ 	$logger->progress( $progressId, $progress->get() );
+
+	Time::HiRes::usleep(250);
+      }
+      
       $_->join for @hobos;
-      say STDERR "        - returning to single-threaded operation";
+      $logger->log( 6, "- returning to single-threaded operation" );
     }
     
     # rename all files in order
@@ -235,52 +305,51 @@ sub animationToStore {
 		      "$animdir/../$animid.mp4" );
 
     File::Path::rmtree( $animdir );
+
+    # all is done
+    $logger->progress( $progressId, 1, "animation @{[$i+1]}/$totalNofBackOrders", 1 );
     
     ### issues:
     # - automatic derivation of dimensions (needs to be multiple of 2)
   }
-
-  return $fullpathid;
 }
+    
 
-
-
-sub imageToStore {
+sub imageFromStore {
   my $self = shift;
   my ( $image ) = @_;
-  return $self->_toStore( 'Images', $image );
+  return $self->_fromStore( 'Images', $image );
 }
 
 
-sub videoToStore {
+
+sub videoFromStore {
   my $self = shift;
   my ( $video ) = @_;
-  return $self->_toStore( 'Videos', $video );
+  return $self->_fromStore( 'Videos', $video );
 }
 
 
-sub iframeToStore {
+sub iframeFromStore {
   my $self = shift;
   my ( $iframe ) = @_;
-  return $self->_toStore( 'Iframes', $iframe );
+  return $self->_fromStore( 'Iframes', $iframe );
 }
 
 
-
-sub audioToStore {
+sub audioFromStore {
   my $self = shift;
   my ( $audio ) = @_;
-  return $self->_toStore( 'Audios', $audio );
+  return $self->_fromStore( 'Audios', $audio );
 }
 
 
-sub _toStore {
+sub _fromStore {
   my $self = shift;
   my ( $type, $file ) = @_;
 
   # find extension
   my ( undef, undef, $ext ) = File::Basename::fileparse( $file, qr/\.[^.]+$/ );
-
   
   # create store id
   my $id = Data::UUID->new();
@@ -290,17 +359,48 @@ sub _toStore {
     #$fullpathid = File::Spec->catfile( $self->{base}, 'media', $type, $id->to_string( $uuid ) . $ext );
     $fullpathid = "$self->{base}/media/$type/@{[$id->to_string( $uuid )]}$ext";
   } until( ! -e $fullpathid );
-  
-  # copy file
-  die( "Error: cannot find media file '$file'\n" ) unless( -r $file );
-  File::Copy::cp( $file, $fullpathid );
 
-  # return store id
+  # register backorder
+  push @{$self->{copyBackOrders}},
+    {
+     type => $type,
+     from => $file,
+     to   => $fullpathid,
+    };
+  
   return $fullpathid;
+
+  # # copy file
+  # die( "Error: cannot find media file '$file'\n" ) unless( -r $file );
+  # File::Copy::cp( $file, $fullpathid );
 }
 
+
+sub processCopyBackOrders {
+  my $self = shift;
+  my ( $progressId ) = @_;
+
+  my $logger = $BeamerReveal::Log::logger;
+  
+  my $totalNofBackOrders = @{$self->{copyBackOrders}};
+  for( my $i = 0; $i < $totalNofBackOrders; ++$i ) {
+    my $bo = $self->{copyBackOrders}->[$i];
+
+    # verify if source file exists
+    $logger->fatal( "Error: cannot find media file '$bo->{from}'\n" ) unless( -r $bo->{from} );
+
+    # report progress and copy
+    $logger->progress( $progressId, $i, "file $i/$totalNofBackOrders", $totalNofBackOrders );
+    File::Copy::cp( $bo->{from}, $bo->{to} );
+  }
+  $logger->progress( $progressId, $totalNofBackOrders, "file $totalNofBackOrders/$totalNofBackOrders", $totalNofBackOrders );
+}
+
+
+
 sub _animWork {
-  my ( $plan, $nofCores, $fileContent, $animdir, $self, $animation, $sliceSize ) = @_;
+  my ( $plan, $nofCores, $fileContent, $animdir, $self, $animation, $sliceSize, $progress, $progressId ) = @_;
+
   my $cmd;
   my $logFile = IO::File->new();
   my $coreId = sprintf( '%0' . nofdigits( $nofCores ) . 'd', $plan->{nr} );
@@ -330,12 +430,48 @@ sub _animWork {
   # run TeX
   $cmd = [ $self->{compiler},
 	   "--output-directory=$animdir", "$texFileName" ];
-  BeamerReveal::IPC::Run::run( $cmd, $coreId, 8 );
 
+  my $logger = $BeamerReveal::Log::logger;
+  my $counter = 0;
+  BeamerReveal::IPC::Run::runsmart( $cmd, 1, qr/\[(\d+)\]/,
+				    sub {
+				      while( scalar @_ ) {
+					my $a = shift @_;
+					while( $a > $counter ) {
+					  ++$counter;
+					  $progress->incr();
+					  if ( $nofCores == 1 ) {
+					    $logger->progress( $progressId, $progress->get() );
+					  }
+					}
+				      }
+				    },
+				    $coreId,
+				    8
+				  );
+  # my $in = '';
+  # my $out;
+  # my $err;
+  # my $logger = $BeamerReveal::Log::logger;
+  # my $h  = harness $cmd, \$in, \$out, \$err;
+  # start $h;
+  # while( $h->pumpable ) {
+  #   pump $h;
+  #   if( $out =~ /(\[\d+\])/ ) {
+  #     $logger->log( 0, "================== $1 ====================\n\n" . $out . "\n" );
+  #     # $logger->progress( $self->{progressId}, $1, "background $1/$2", $2 );
+  #     $out = '';
+  #   };
+  # }
+  # finish $h or die( "returned $?" );
+  
+  #BeamerReveal::IPC::Run::run( $cmd, $coreId, 8 );
+  
   say $logFile "- Cropping PDF file";
   # run pdfcrop
   $cmd = [ $self->{pdfcrop}, '--margins', '-2', "animation-$coreId.pdf" ];
   BeamerReveal::IPC::Run::run( $cmd, $coreId, 8, $animdir );
+  $progress->incr();
 
   # run pdftoppm
   my $xrange = 2 * int( $self->{presentationparameters}->{canvaswidth} * $animation->{width} );
@@ -347,6 +483,7 @@ sub _animWork {
 	   '-scale-to-y', "$yrange",
 	   "animation-$coreId-crop.pdf", "./frame-$coreId", '-jpeg' ];
   BeamerReveal::IPC::Run::run( $cmd, $coreId, 8, $animdir );
+  $progress->incr();
 
   # correct for too short slicesize in filenames coming from pdftoppm
   say $logFile "- Cleaning up jpg files";
@@ -362,6 +499,8 @@ sub _animWork {
     }
   }
   $logFile->close();
+
+  $progress->incr();
 }
 
     
@@ -379,7 +518,7 @@ BeamerReveal::MediaManager - MediaManager
 
 =head1 VERSION
 
-version 20251227.1426
+version 20251230.2042
 
 =head1 SYNOPSIS
 
@@ -397,7 +536,7 @@ we reuse the generated video file. If the source data has changed, we regenerate
 
 =head2 new()
 
-  $mm = BeamerReveal::MediaManager->new( $jobname, $base, $presoparams )
+  $mm = BeamerReveal::MediaManager->new( $jobname, $base, $presoparams, $id )
 
 The constructor sets up the manager.
 
@@ -425,7 +564,7 @@ presentation
 
 =item . C<$mm>
 
-mediamanager object that can be used to manage the media of the presentation.
+return value: the mediamanager object
 
 =back
 
@@ -439,7 +578,7 @@ Fetches the original reveal support files and copies them into the media store.
 
   $path = $mm->slideFromStore( $slide )
 
-Fetches the media store pathname of $slide. Slides have been entered into the store by the
+Fetches the media store pathname of $slide. Slides are entered into the store by the
 C<Frameconverter>.
 
 =over 4
@@ -454,13 +593,11 @@ the path of the slide (in the media store)
 
 =back
 
-=head2 animationToStore()
+=head2 animationFromStore()
 
-  $path = $mm->animationToStore( $animation )
+  $path = $mm->animationFromStore( $animation )
 
-Generates the animation with content C<$animation> in the media store and returns the media store
-path to the animation. The generation is done in parallel using multithreading. If the method fails
-the temporary files are kept, otherwise they are removed.
+Returns the media store path to the animation. If the animation is not yet constructed, it will be put in back order, such that it can be generated later by C<processConstructionBackOrders()>.
 
 =over 4
 
@@ -474,12 +611,28 @@ the path of the animation (in the media store)
 
 =back
 
-=head2 imageToStore()
+=head2 processConstructionBackOrders()
 
-  $path = $mm->imageToStore( $image )
+  $mm->processConnstructionBackOrders( $id )
 
-Copies the C<$image> to the store under a unique ID and returns that ID (the filename of the oject
-in the media store).
+Generates all animations if they are not cached in the store.
+The generation is done in parallel using multithreading. If the method fails
+the temporary files are kept, otherwise they are removed. On MS-Windows there
+is no working multithreading/multiprocessing.
+
+=over 4
+
+=item . C<$id>
+
+progress ID (controls the corresonding progressbar)
+
+=back
+
+=head2 imageFromStore()
+
+  $path = $mm->imageFromStore( $image )
+
+Fetches the unique ID of the C<$image> and returns that ID (the filename of the oject in the media store). The object is put in back-order such that it can be copied later, using C<processCopyBackOrders>.
 
 =over 4
 
@@ -493,12 +646,11 @@ the path to the image (in the media store)
 
 =back
 
-=head2 videoToStore()
+=head2 videoFromStore()
 
-  $path = $mm->videoToStore( $video )
+  $path = $mm->videoFromStore( $video )
 
-Copies the C<$video> to the store under a unique ID and returns that ID (the filename of the oject
-in the media store).
+Fetches the unique ID of the C<$video> and returns that ID (the filename of the oject in the media store). The object is put in back-order such that it can be copied later, using C<processCopyBackOrders>.
 
 =over 4
 
@@ -512,12 +664,11 @@ the path to the video (in the media store)
 
 =back
 
-=head2 iframeToStore()
+=head2 iframeFromStore()
 
-  $path = $mm->iframeToStore( $iframe )
+  $path = $mm->iframeFromStore( $iframe )
 
-Copies the C<$iframe> to the store under a unique ID and returns that ID (the filename of the oject
-in the media store).
+Fetches the unique ID of the C<$iframe> and returns that ID (the filename of the oject in the media store). The object is put in back-order such that it can be copied later, using C<processCopyBackOrders>.
 
 =over 4
 
@@ -531,12 +682,11 @@ the path to the iframe (in the media store)
 
 =back
 
-=head2 audioToStore()
+=head2 audioFromStore()
 
-  $path = $mm->audioToStore( $audio )
+  $path = $mm->audioFromStore( $audio )
 
-Copies the C<$audio> to the store under a unique ID and returns that ID (the filename of the oject
-in the media store).
+Fetches the unique ID of the C<$audio> and returns that ID (the filename of the oject in the media store). The object is put in back-order such that it can be copied later, using C<processCopyBackOrders>.
 
 =over 4
 
@@ -550,9 +700,27 @@ the path to the audio (in the media store)
 
 =back
 
-=head2 _toStore()
+=head2 _fromStore()
 
-Do not use directly.
+Helper function; do not use directly.
+
+=head2 processCopyBackOrders()
+
+  $mm->processCopyBackOrders( $id )
+
+Copies the backordered files from their original location into the store, based on the backorder list.
+
+=over 4
+
+=item . C<$id>
+
+progress ID (controls the corresponding progressbar)
+
+=back
+
+=head2 _animWork()
+
+Worker function; do not use directly.
 
 =head1 AUTHOR
 
